#include <raylib.h>

#include "Panda.hpp"


namespace panda {
  void Sprite::SetPivot(Vector2 p)
  {
    pivotX = p.x;
    pivotY = p.y;
    CalculatePivotFlip();
  }

  void Sprite::SetPivotRatio(Vector2 ratio)
  {
    pivotX = texture->width*ratio.x;
    pivotY = texture->height*ratio.y;
    CalculatePivotFlip();
  }

  void Sprite::SetFlip(bool x, bool y)
  {
    flipX = x;
    flipY = y;
    CalculatePivotFlip();
  }

  void Sprite::SetFlipX(bool x)
  {
    flipX = x;
    CalculatePivotFlip();
  }

  void Sprite::SetFlipY(bool y)
  {
    flipY = y;
    CalculatePivotFlip();
  }

  Vector2 Sprite::GetPivot() const
  {
    return {pivotX, pivotY};
  }

  Vector2 Sprite::GetPivotFlip() const
  {
    return {pivotX_flip, pivotY_flip};
  }

  bool Sprite::GetFlipX() const
  {
    return flipX;
  }

  bool Sprite::GetFlipY() const
  {
    return flipY;
  }

  void Sprite::CalculatePivotFlip()
  {
    pivotX_flip = flipX ? texture->width-pivotX : pivotX;
    pivotY_flip = flipY ? texture->height-pivotY : pivotY;
  }

  void DrawSprite(const panda::Sprite &sprite, const float &x, const float &y, const Color &color)
  {
    DrawSpriteEx(sprite, {x, y}, {1.0f, 1.0f}, 0.0f, color);
  }

  void DrawSpriteV(const panda::Sprite &sprite, const Vector2 &pos, const Color &color)
  {
    DrawSpriteEx(sprite, pos, {1.0f, 1.0f}, 0.0f, color);
  }

  void DrawSpriteEx(const panda::Sprite &sprite, const Vector2 &pos, const Vector2 &scale, const float &rotation, const Color &color)
  {
    Vector2 dest = GetPointInSprite({0.0f, 0.0f}, sprite, pos, scale, rotation);

    DrawTexturePro(
      *sprite.texture,
      panda::GetTextureRect(*sprite.texture, sprite.GetFlipX(), sprite.GetFlipY()),
      {
        dest.x, dest.y,
        scale.x*sprite.texture->width,
        scale.y*sprite.texture->height
      },
      {0.0f, 0.0f},
      rotation, color
    );
  }

  Vector2 GetPointInSprite(const Vector2 &point, const panda::Sprite &sprite, const Vector2 &pos, const Vector2 &scale, const float &rotation)
  {
    Vector2 pivot_temp = (sprite.GetFlipX() || sprite.GetFlipY()) ? sprite.GetPivotFlip() : sprite.GetPivot();

    panda::Matrix matrix;
    matrix.translate(-pivot_temp.x, -pivot_temp.y);
    matrix.translate(point.x, point.y);
    matrix.scale(scale.x, scale.y);
    matrix.rotate(rotation * DEG2RAD);
    matrix.translate(pos.x, pos.y);

    return { matrix.tx, matrix.ty };
  }

}