#pragma once

#include <cstddef>
#include <list>
#include <vector>
#include <functional>

#include "utils/Random.hpp"

namespace panda
{

  // template<typename T>
  // class Pool
  // {
  // public:
  //     Pool(int size) : container(size) { }

  //     T &get()
  //     {
  //         if (activeIndex != container.size())
  //         {
  //             activeIndex++;
  //             return container[activeIndex-1];
  //         }
  //         else
  //         {
  //             if (custom)
  //             {
  //                 return container[custom()];
  //             }
  //             else
  //             {
  //                 int r = Random::getInstance().UNI() * (container.size()-1);
  //                 return container[r];
  //             }
  //         }
  //     }

  //     void put(int index)
  //     {
  //         activeIndex--;
  //         std::swap(container[activeIndex], container[index]);
  //     }

  //     // int countActive() { return activeIndex; }
  //     int activeIndex;
  //     std::vector<T> container;
  //     std::function<int()> custom;
  // };

  template<typename T, size_t SIZE>
  class PoolPointer {
   private:
    std::list<T*> free;

   public:
    PoolPointer() : free()
    {
      for (size_t i=0; i<SIZE; i++) free.emplace_back(new T());
    }

    T* get() {
      if (free.empty())
        return new T(); // no objects are free
      
      // get the last object in the list
      auto obj = free.back();
      free.pop_back();
      return obj;
    }

    int getFreeSize()
    {
      return free.size();
    }

    /// Mark the given object for reuse in the future.
    inline void put(T* obj) { free.emplace_back(obj); }
  };

  template<typename T, size_t SIZE>
  class PoolCircular {
    public:
      T pool[SIZE];

      PoolCircular() :
        head(0)
      { }

      T *Get()
      {
        head = (head + 1) % SIZE;
        return pool[head];
      }

      size_t size() { return SIZE; }
      int getHead() { return head; } // for debug only!!!
    
    private:
      int head;
  };
}